<?php

namespace Drupal\config_enforce_devel;

use Drupal\Core\File\FileSystemInterface;
use Drupal\config_enforce\ConfigEnforce;

/**
 * Enforces configuration.
 */
class ConfigEnforceDevel extends ConfigEnforce {

  /* The name of a config object to enforce. */
  protected $config;

  /* The enforcement level for a config object. */
  protected $level;

  /* The module in which to write a config file. */
  protected $module;

  /* The directory in which to write a config file. */
  protected $directory;

  /* The path to write a config file. */
  protected $path;

  /* The URI of the host form. */
  protected $uri = '';

  /**
   * Provide Config Devel module with configs to read from disk.
   *
   * @see hook_config_devel_auto_import()
   */
  public static function autoImport() {
    $auto_import = [];
    foreach (self::getEnforcedConfigs() as $name => $config) {
      $auto_import[$name] = [
        'filename' => $config['path'],
        'hash' => isset($config['hash']) ? $config['hash'] : '',
      ];
    }
    return $auto_import;
  }

  /**
   * Provide Config Devel module with config files to write to disk.
   *
   * @see hook_config_devel_auto_export()
   */
  public static function autoExport() {
    $auto_export = [];
    foreach (self::getEnforcedConfigs() as $name => $config) {
      $auto_export[] = $config['path'];
    }
    return $auto_export;
  }

  /**
   * Save hashes of auto-imported config files.
   *
   * @see config_enforce_devel_config_devel_post_auto_import()
   */
  public static function postAutoImport(array $auto_import) {
    $enforced_configs = self::getEditableEnforcedConfigs();
    foreach ($auto_import as $name => $info) {
      $config = $enforced_configs->get(self::encode($name));
      $config['hash'] = $info['hash'];
      $enforced_configs->set(self::encode($name), $config);
    }
    $enforced_configs->save();
  }

  /**
   * Set the config object name attribute.
   */
  public function setConfig(string $config) {
    $this->config = $config;
    return $this;
  }

  /**
   * Set the enforcement level attribute.
   */
  public function setLevel(string $level) {
    $this->level = $level;
    return $this;
  }

  /**
   * Set the target module attribute.
   */
  public function setModule(string $module) {
    $this->module = $module;
    return $this;
  }

  /**
   * Set the config directory attribute.
   */
  public function setDirectory(string $directory) {
    $this->directory = $directory;
    return $this;
  }

  /**
   * Set the config directory attribute.
   */
  public function setUri(string $uri) {
    $this->uri = $uri;
    return $this;
  }

  /**
   * Remove a config object from the enforced configs.
   */
  public function clear() {
    self::getEditableEnforcedConfigs()
      ->clear(self::encode($this->config))
      ->save();
    return $this;
  }

  /**
   * Add or update a config object in the enforced configs.
   */
  public function save() {
    $this
      ->setPath()
      ->prepareConfigDirectory()
      ->initializeNewConfigFile()
      ->cleanStaleConfigFile();

    self::getEditableEnforcedConfigs()
      ->set(self::encode($this->config), [
        'module' => $this->module,
        'directory' => $this->directory,
        'path' => $this->path,
        'level' => $this->level,
        'uri' => $this->uri,
      ])
      ->save();
    return $this;
  }

  /**
   * Return this module's stored config.
   */
  protected static function getEditableEnforcedConfigs() {
    return \Drupal::configFactory()
      ->getEditable(self::CONFIG_NAME);
  }

  /**
   * Delete an existing config file, if its path has changed.
   */
  protected function cleanStaleConfigFile() {
    if ($this->configFilePathChanged()) {
      \Drupal::service('file_system')
        ->delete(self::getPath($this->config));
    }
    return $this;
  }

  /**
   * Create a new config file.
   *
   * Config Devel is generally responsible for writing enforced config to disk.
   * However, when we enable a config for the first time, it won't yet be in
   * the list of enforced configs the Config Devel should write to disk. So, we
   * have to write it ourselves this time.
   *
   * @see ConfigEnforceDevel::autoExport()
   */
  protected function initializeNewConfigFile() {
    if ($this->configFilePathChanged()) {
      \Drupal::service('config_devel.writeback_subscriber')
        ->writeBackConfig(\Drupal::config($this->config), [$this->path]);
    }
    return $this;
  }

  /**
   * Determine whether the path for a config file has changed (or is new).
   */
  protected function configFilePathChanged() {
    $old_path = ConfigEnforce::getPath($this->config);
    if (!file_exists($old_path)) return TRUE;
    return $this->path !== $old_path;
  }

  /**
   * Ensure that the directory, in which a config file should be written, exists.
   */
  protected function prepareConfigDirectory() {
    $file_system = \Drupal::service('file_system');
    $directory = $file_system->dirname($this->path);
    $options = FileSystemInterface::CREATE_DIRECTORY + FileSystemInterface::MODIFY_PERMISSIONS;
    $file_system->prepareDirectory($directory, $options);
    return $this;
  }

  /**
   * Derive the config file path from the target module and config directory.
   */
  protected function setPath() {
    $this->path = drupal_get_path('module', $this->module);
    $this->path .= '/' . $this->directory . '/';
    $this->path .= $this->config . '.yml';
    return $this;
  }

}
