<?php

namespace Drupal\config_enforce_devel\Form;

use Drupal\config_enforce\ConfigEnforce;
use Drupal\config_enforce\Form\FormHelperTrait;
use Drupal\config_enforce_devel\ConfigEnforceDevel;
use Drupal\Core\Config\FileStorage;
use Drupal\Core\Config\InstallStorage;
use Drupal\Core\Config\StorageInterface;
use Drupal\Core\Form\FormStateInterface;

/**
 * Defines a settings form for the Config Enforce Devel module.
 */
trait DevelFormHelperTrait {

  use FormHelperTrait;

  // The current config object name.
  protected $current_config;

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->setSharedFormProperties($form, $form_state);

    $this->saveConfig();

    return parent::submitForm($this->form(), $this->formState());
  }

  /**
   * Add fields for a single config object.
   */
  protected function addEnforcedConfigItem($config) {
    parent::addEnforcedConfigItem($config);

    $this
      ->setCurrentConfig($config)
      ->addConfigEnforceEnabledField()
      ->overrideTargetModuleField()
      ->overrideConfigDirectoryField()
      ->overrideEnforcementLevelField()
      ->setConfigEnforcePathStates();
  }

  /**
   * Add a checkbox to control whether the current config object shouldbe enforced.
   */
  protected function addConfigEnforceEnabledField() {
    $this->form()['config_enforce'][$this->getCurrentConfig()]['config_enforce_enabled'] = [
      '#type' => 'checkbox',
      '#title' => 'Enforce config',
      '#description' => "Enable enforcement of the config from this form.",
      '#default_value' => $this->currentConfigIsEnforced(),
      '#attributes' => [
        'config-enforce-enabled' => $this->getEnabledAttribute(),
      ],
    ];
    return $this;
  }

  /**
   * Add a field to allow the target module to be selected.
   */
  protected function overrideTargetModuleField() {
    $config = $this->getCurrentConfig();
    $this->form()['config_enforce'][$config]['target_module'] = $this->getTargetModuleField();
    $this->form()['config_enforce'][$config]['target_module']['#states'] = $this->getEnabledDependentState();
    if (ConfigEnforce::isEnforced($config)) {
      $module = ConfigEnforce::getModule($config);
      $this->form()['config_enforce'][$config]['target_module']['#title'] .= ' (<em>' . $module . '</em>)';
      $this->form()['config_enforce'][$config]['target_module']['#default_value'] = $module;
    }
    return $this;
  }

  /**
   * Add a field to allow the config directory to be selected.
   */
  protected function overrideConfigDirectoryField() {
    $config = $this->getCurrentConfig();
    $this->form()['config_enforce'][$config]['config_directory'] = $this->getConfigDirectoryField();
    $this->form()['config_enforce'][$config]['config_directory']['#states'] = $this->getEnabledDependentState();
    if (ConfigEnforce::isEnforced($config)) {
      $directory = ConfigEnforce::getDirectory($config);
      $this->form()['config_enforce'][$config]['config_directory']['#title'] .= ' (<em>' . $directory . '</em>)';
      $this->form()['config_enforce'][$config]['config_directory']['#default_value'] = $directory;
    }
    return $this;
  }

  /**
   * Add a field to allow the enforcement level directory to be specified.
   */
  protected function overrideEnforcementLevelField() {
    $config = $this->getCurrentConfig();
    $this->form()['config_enforce'][$config]['enforcement_level'] = $this->getEnforcementLevelField();
    $this->form()['config_enforce'][$config]['enforcement_level']['#states'] = $this->getEnabledDependentState();
    if ($this->currentConfigIsEnforced()) {
      $this->form()['config_enforce'][$config]['enforcement_level']['#title'] .= ' (<em>' . ConfigEnforce::getLevelLabel($config) . '</em>)';
      $this->form()['config_enforce'][$config]['enforcement_level']['#default_value'] = ConfigEnforce::getLevel($config);
    }
    return $this;
  }

  /**
   * Set the current config object name.
   */
  protected function setCurrentConfig($config) {
    $this->current_config = $config;
    return $this;
  }

  /**
   * Return the current config object name.
   */
  protected function getCurrentConfig() {
    return $this->current_config;
  }

  /**
   * Set the Form API '#states' setting for the config file path.
   */
  protected function setConfigEnforcePathStates() {
    if ($this->currentConfigIsEnforced()) {
      $this->form()['config_enforce'][$this->getCurrentConfig()]['config_enforce_path']['#states'] = $this->getEnabledDependentState();
    }
    return $this;
  }

  /**
   * Determine whether the current config object is being enforced.
   */
  protected function currentConfigIsEnforced() {
    return ConfigEnforce::isEnforced($this->getCurrentConfig());
  }

  /**
   * Return the config-specific "enabled" attribute value.
   *
   * @see foo()
   * @see getEnabledDependentState($config)
   */
  protected function getEnabledAttribute() {
    return 'config_enforce_enabled-' . $this->getCurrentConfig();
  }

  /**
   * Return the Form API '#states' setting for config fields.
   *
   * This makes a given config's fields' visibility depend on the 'enabled' field for that config.
   */
  protected function getEnabledDependentState() {
    return [
      'invisible' => [
        ':input[config-enforce-enabled="' . $this->getEnabledAttribute() . '"]' => ['checked' => FALSE],
      ],
    ];
  }

  /**
   * Save our config.
   */
  protected function saveConfig() {
    $this->configEnforce = new ConfigEnforceDevel();
    foreach ($this->getFormConfigs() as $config) {
      $this->configEnforce->setConfig($config);

      if (!$this->getEnforced($config)) {
        $this->configEnforce->clear();
        continue;
      }

      $this->prepareConfig($config);

      $this->configEnforce->save();
    }
  }

  /**
   * Prepare our config.
   */
  protected function prepareConfig($config) {
    $this->configEnforce
      ->setModule($this->getModule($config))
      ->setDirectory($this->getDirectory($config))
      ->setLevel($this->getLevel($config));
  }

  /**
   * Return a list of modules.
   */
  protected function getAllModules() {
    $modules = [];
    $moduleHandler = \Drupal::moduleHandler();
    foreach ($moduleHandler->getModuleList() as $name => $module) {
      $modules[$name] = $moduleHandler->getName($name);
    }
    asort($modules);
    return $modules;
  }

  /**
   * Return a list of available modules to target for writing config.
   */
  protected function getAvailableModules() {
    $modules = [];
    $available_modules = $this->config('config_enforce_devel.settings')->get('available_modules');
    $available_modules = !is_null($available_modules) ? $available_modules : [];
    foreach ($available_modules as $module) {
      $modules[$module] = $this->getAllModules()[$module];
    }
    return $modules;
  }

  /**
   * Return a human-readable name for a given module.
   */
  protected function getModuleName($module) {
    return $this->getAvailableModules()[$module];
  }

  /**
   * Return a list of directories for writing config.
   */
  protected function getAvailableDirectories() {
    // @TODO Look this up in the storage service or make this a hook.
    return [
      InstallStorage::CONFIG_INSTALL_DIRECTORY => 'Install',
      InstallStorage::CONFIG_OPTIONAL_DIRECTORY => 'Optional',
    ];
  }

  /**
   * Return a human-readable name for a given config directory.
   */
  protected function getDirectoryLabel($directory) {
    return $this->getAvailableDirectories()[$directory];
  }

  /**
   * Return a list of all config object names from given module and directory.
   */
  protected function getModuleConfigNames($module, $directory) {
    $module_config_path = drupal_get_path('module', $module) . '/' . $directory;
    $storage = new FileStorage($module_config_path, StorageInterface::DEFAULT_COLLECTION);
    return $storage->listAll();
  }

  /**
   * Return array of config enforce values from the form.
   */
  protected function getConfigEnforceValuesFromForm() {
    return $this->formState()->getValue('config_enforce');
  }

  /**
   * Return the value of a specific field for a given config object.
   */
  protected function getConfigEnforceValue(string $config, string $field) {
    $values = $this->getConfigEnforceValuesFromForm();
    if ($this->areConfigEnforceValuesWrapped()) {
      // Some forms appear to wrap our settings in an array.
      $values = $values['config_enforce'];
    }
    return $values[$config][$field];
  }

  /**
   * Determine whether config enforce values are wrapped in an extra array.
   */
  protected function areConfigEnforceValuesWrapped() {
    $values = $this->getConfigEnforceValuesFromForm();
    return array_key_exists('config_enforce', $values);
  }

  /**
   * Return the new target module from the form state.
   */
  protected function getModule($config) {
    return $this->getConfigEnforceValue($config, 'target_module');
  }

  /**
   * Return the new config directory from the form state.
   */
  protected function getDirectory($config) {
    return $this->getConfigEnforceValue($config, 'config_directory');
  }

  /**
   * Return the new enabled setting from the form state.
   */
  protected function getEnforced($config) {
    return $this->getConfigEnforceValue($config, 'config_enforce_enabled');
  }

  /**
   * Return the new enforcement level from the form state.
   */
  protected function getLevel($config) {
    return $this->getConfigEnforceValue($config, 'enforcement_level');
  }

  /**
   * Return the basic definition for a target module field.
   */
  protected function getTargetModuleField() {
    return [
      '#type' => 'select',
      '#title' => $this->t('Target module'),
      '#description' => "The module to which config should be written.",
      '#options' => $this->getAvailableModules(),
      '#default_value' => $this->config('config_enforce_devel.settings')->get('defaults')['target_module'],
    ];
  }

  /**
   * Return the basic definition for a config directory field.
   */
  protected function getConfigDirectoryField() {
    return [
      '#type' => 'select',
      '#title' => $this->t('Directory'),
      '#description' => "The config directory to which config should be written.",
      '#options' => $this->getAvailableDirectories(),
      '#default_value' => $this->config('config_enforce_devel.settings')->get('defaults')['config_directory'],
    ];
  }

  /**
   * Return the basic definition for an enforcement level field.
   */
  protected function getEnforcementLevelField() {
    return [
      '#type' => 'select',
      '#title' => $this->t('Enforcement level'),
      '#description' => "The level of enforcement to apply to config objects.",
      '#options' => ConfigEnforce::getLevels(),
      '#default_value' => $this->config('config_enforce_devel.settings')->get('defaults')['enforcement_level'],
    ];
  }

  /**
   * List unenforced config object names that have not been flagged for ignoring.
   */
  protected function getUnenforcedConfigs() {
    $unenforced_configs = $this->getAllUnenforcedConfigs();
    $ignored_configs = $this->getIgnoredConfigs();
    $filtered_configs = array_diff($unenforced_configs, $ignored_configs);
    return array_combine($filtered_configs, $filtered_configs);
  }

  /**
   * List all unenforced config object names.
   */
  protected function getAllUnenforcedConfigs() {
    $active_storage_configs = $this->configFactory()->listAll();
    $enforced_configs = array_keys(ConfigEnforce::getEnforcedConfigs());
    $unenforced_configs = array_diff($active_storage_configs, $enforced_configs);
    return array_combine($unenforced_configs, $unenforced_configs);
  }

  /**
   * List ignored config objects.
   */
  protected function getIgnoredConfigs() {
    $ignored_configs = $this->config('config_enforce_devel.settings')->get('ignored_configs');
    return is_null($ignored_configs) ? [] : array_combine($ignored_configs, $ignored_configs);
  }

}
