<?php

namespace Drupal\config_enforce_devel\Form;

use Drupal\Component\Utility\Html;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Drupal\config_enforce\ConfigEnforce;
use Drupal\config_enforce\Form\FormHelperTrait;

/**
 * Form that displays all the config variables to edit them.
 */
class EnforcedConfigs extends FormBase {

  use FormHelperTrait;

  const FORM_ID = 'config_enforce_enforced_configs_form';

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $this->setSharedFormProperties($form, $form_state);

    $this->form()['config_objects'] = [
      '#type' => 'table',
      '#header' => $this->getHeader(),
      '#rows' => $this->getRows(),
      '#empty' => $this->t('No config objects found'),
    ];

    $this->form()['#attached']['library'][] = 'config_enforce_devel/config-enforce-devel';
    $this->form()['#attached']['library'][] = 'core/drupal.dialog.ajax';

    return $this->form();
  }

  /**
   * {@inheritdoc}
   *
   * This provides an easy way for enforced configs to be enforced themselves.
   */
  public function getEditableConfigNames() {
    return ['config_enforce.enforced_configs'];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {}

  /**
   * Return a table header render array.
   */
  protected function getHeader() {
    return [
      'name' => ['data' => $this->t('Configuration object name')],
      'level' => ['data' => $this->t('Enforcement level')],
      'module' => ['data' => $this->t('Target module')],
      'directory' => ['data' => $this->t('Config directory')],
      'settings' => ['data' => $this->t('Settings')],
    ];
  }

  /**
   * Return a table rows render array.
   */
  protected function getRows() {
    $rows = [];
    foreach ($this->getEnforcedConfigs() as $config_name) {
      $settings = [
        'data' => [
          '#type' => 'link',
          '#title' => $this->t('Edit'),
          '#url' => Url::fromRoute('config_enforce_devel.config_edit', ['arg' => $config_name]),
          '#attributes' => [
            'class' => [
              'use-ajax',
              'button',
            ],
          ],
        ],
      ];

      $config_display_name = $config_name;
      $uri = ConfigEnforce::getUri($config_name);
      if (!empty($uri)) {
        $config_display_name = Link::fromTextAndUrl($config_name, Url::fromUri('internal:' . $uri));
      }

      $rows[] = [
        'class' => $this->getRowClasses($config_name),
        'data' => [
          'name' => $config_display_name,
          'level' => ConfigEnforce::getLevelLabel($config_name),
          'module' => ConfigEnforce::getModule($config_name),
          'directory' => ConfigEnforce::getDirectory($config_name),
          'settings' => $settings,
        ],
      ];
    }

    return $rows;
  }

  protected function getRowClasses($config_name) {
    $classes = [];
    if (ConfigEnforce::isEnforced($config_name)) {
      $classes[] = 'enforced';
      $classes[] = 'level-' . ConfigEnforce::getLevel($config_name);
    }
    return $classes;
  }

  /**
   * List only enforced config object names.
   */
  protected function getEnforcedConfigs() {
    $enforced = [];
    foreach (ConfigEnforce::getEnforcedConfigs() as $name => $info) {
      $enforced[] = $name;
    }
    return $enforced;
  }

}
